//!
//!
use super::{models, schema::member_referrers};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of member_referrers").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_resource: &models::NewMemberReferrer,
  ) -> Result<uuid::Uuid, Error> {
    use member_referrers::dsl;

    let inserted = diesel::insert_into(member_referrers::table)
      .values(new_resource)
      .returning(dsl::id)
      .get_result(conn)?;

    Ok(inserted)
  }

  ///
  pub fn inserts(
    &self,
    conn: &diesel::PgConnection,
    new_referrers: &Vec<models::NewMemberReferrer>,
  ) -> Result<Vec<uuid::Uuid>, Error> {
    use member_referrers::dsl;

    let inserted = diesel::insert_into(member_referrers::table)
      .values(new_referrers)
      .returning(dsl::id)
      .get_results(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::MemberReferrer>, Error> {
    match member_referrers::table
      .find(id as uuid::Uuid)
      .first::<models::MemberReferrer>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::MemberReferrer>, Error> {
    use member_referrers::dsl;

    match member_referrers::table
      .filter(dsl::member_id.eq(member_id))
      .first::<models::MemberReferrer>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = member_referrers::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_referrers::dsl::member_id.eq(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(member_referrers::dsl::can_use.eq(sp));
      }
      if let Some(sp) = &s.memo_like {
        q = q.filter(member_referrers::dsl::memo.like(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::MemberReferrer>, Error> {
    let mut q = member_referrers::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        q = q.filter(member_referrers::dsl::member_id.eq(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(member_referrers::dsl::can_use.eq(sp));
      }
      if let Some(sp) = &s.memo_like {
        q = q.filter(member_referrers::dsl::memo.like(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "member_id" => {
              q = q.order_by(member_referrers::member_id.asc());
            }
            "can_use" => {
              q = q.order_by(member_referrers::can_use.asc());
            }
            "created_at" => {
              q = q.order_by(member_referrers::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(member_referrers::updated_at.asc());
            }
            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "member_id" => {
              q = q.order_by(member_referrers::member_id.desc());
            }
            "can_use" => {
              q = q.order_by(member_referrers::can_use.desc());
            }
            "created_at" => {
              q = q.order_by(member_referrers::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(member_referrers::updated_at.desc());
            }

            _ => {}
          },
        };
      }
    }

    q.load::<models::MemberReferrer>(conn)
  }

  ///
  pub fn update(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifyMemberReferrer,
  ) -> Result<u64, Error> {
    use member_referrers::dsl;

    diesel::update(dsl::member_referrers.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: uuid::Uuid) -> Result<u64, Error> {
    use member_referrers::dsl;

    diesel::delete(member_referrers::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn deletes(&self, conn: &diesel::PgConnection, ids: Vec<uuid::Uuid>) -> Result<u64, Error> {
    use member_referrers::dsl;

    diesel::delete(member_referrers::table.filter(dsl::id.eq_any(ids)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
