//!
//!
use super::{models, schema::sites};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of sites").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_member: &models::NewSite,
  ) -> Result<models::Site, Error> {
    let inserted = diesel::insert_into(sites::table)
      .values(new_member)
      .get_result::<models::Site>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::Site>, Error> {
    match sites::table.find(id).first::<models::Site>(conn) {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_url(
    &self,
    conn: &diesel::PgConnection,
    url: &str,
  ) -> Result<Option<models::Site>, Error> {
    use sites::dsl;

    match sites::table
      .filter(dsl::url.eq(url))
      .first::<models::Site>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = sites::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.url_like {
        q = q.filter(sites::dsl::url.like(sp));
      }
      if let Some(sp) = &s.name_like {
        q = q.filter(sites::dsl::name.like(sp));
      }
      if let Some(sp) = &s.path_like {
        q = q.filter(sites::dsl::path.like(sp));
      }
      if let Some(sp) = s.show {
        q = q.filter(sites::dsl::show.eq(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(sites::dsl::can_use.eq(sp));
      }
      if let Some(sp) = &s.memo_like {
        q = q.filter(sites::dsl::memo.like(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::Site>, Error> {
    let mut q = sites::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.url_like {
        q = q.filter(sites::dsl::url.like(sp));
      }
      if let Some(sp) = &s.name_like {
        q = q.filter(sites::dsl::name.like(sp));
      }
      if let Some(sp) = &s.path_like {
        q = q.filter(sites::dsl::path.like(sp));
      }
      if let Some(sp) = s.show {
        q = q.filter(sites::dsl::show.eq(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(sites::dsl::can_use.eq(sp));
      }
      if let Some(sp) = &s.memo_like {
        q = q.filter(sites::dsl::memo.like(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "url" => {
              q = q.order_by(sites::url.asc());
            }
            "name" => {
              q = q.order_by(sites::name.asc());
            }
            "path" => {
              q = q.order_by(sites::path.asc());
            }
            "expires_at" => {
              q = q.order_by(sites::expires_at.asc());
            }
            "created_at" => {
              q = q.order_by(sites::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(sites::updated_at.asc());
            }
            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "url" => {
              q = q.order_by(sites::url.desc());
            }
            "name" => {
              q = q.order_by(sites::name.desc());
            }
            "path" => {
              q = q.order_by(sites::path.desc());
            }
            "expires_at" => {
              q = q.order_by(sites::expires_at.desc());
            }
            "created_at" => {
              q = q.order_by(sites::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(sites::updated_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::Site>(conn)
  }

  ///
  pub fn update(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifySite,
  ) -> Result<u64, Error> {
    use sites::dsl;

    diesel::update(dsl::sites.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: uuid::Uuid) -> Result<u64, Error> {
    use sites::dsl;

    diesel::delete(sites::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
